#include <avr/io.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include "s1d1565.h"
#include "font6x8s.c"

#define Reset	3
#define	CS		4
#define A0		5
#define RW		6
#define Enable	7

#define LCDP	PORTA	//Port fr D0-7
#define LCDPI	PINA
#define LCDDD	DDRA

#define LCD		PORTC	//Port fr Steuerleitungen
#define LCDCDD	DDRC

void lcd_writecom (unsigned char byte)
   {cbi(LCD,A0);
	LCDP=byte;
	cbi(LCD,CS);
	cbi(LCD,RW);	
    asm ("nop");
	sbi(LCD,Enable);
    asm ("nop");
    asm ("nop");
    asm ("nop");
    asm ("nop");
	cbi(LCD,Enable);
	sbi(LCD,CS);}

void lcd_writebyte (unsigned char byte)
   {sbi(LCD,A0);
	LCDP=byte;
	cbi(LCD,CS);
	cbi(LCD,RW);	
    asm ("nop");
	sbi(LCD,Enable);
    asm ("nop");
    asm ("nop");
    asm ("nop");
    asm ("nop");
	cbi(LCD,Enable);
	sbi(LCD,CS);}

unsigned char lcd_readbyte (void)
{unsigned char byte;
	sbi(LCD,A0);
	LCDP=255;
	LCDDD=0;
	cbi(LCD,CS);
	sbi(LCD,RW);	
    asm ("nop");
	sbi(LCD,Enable);
    asm ("nop");
    asm ("nop");
    asm ("nop");
    asm ("nop");
	cbi(LCD,Enable);
	sbi(LCD,CS);
    asm ("nop");
    asm ("nop");
    asm ("nop");
	cbi(LCD,CS);
    asm ("nop");
	sbi(LCD,Enable);
    asm ("nop");
    asm ("nop");
    asm ("nop");
    asm ("nop");
	byte=LCDPI;
	cbi(LCD,Enable);
	sbi(LCD,CS);
	LCDDD=255;
	return byte;}

void lcd_stringr (unsigned char *s)         //Display String from RAM
{unsigned char i;
	while (*s) 
		for (i=0; i<6; i++)
			lcd_writebyte(pgm_read_byte(&Font[*s++][i]));
}

void lcd_writechar (unsigned char byte)     //Display char
{unsigned char i;
		for (i=0; i<6; i++)
			lcd_writebyte(pgm_read_byte(&Font[byte][i]));
}

void lcd_string (unsigned const char *progmem_s)     //Display String from Flash
{unsigned char i,c;
	while ((c = pgm_read_byte(progmem_s++)))
		for (i=0; i<6; i++)
			lcd_writebyte(pgm_read_byte(&Font[c][i]));
}

void lcd_init(void)         //Initialize LCD
   {unsigned int i;
	LCDDD=255;
	LCDCDD=(1<<A0)|(1<<Reset)|(1<<Enable)|(1<<RW)|(1<<CS);
	cbi(LCD,Reset);
	for (i=0; i<1000; i++)
		asm("nop");
	sbi(LCD,Reset);
	for (i=0; i<10000; i++)
		asm("nop");
	lcd_writecom(0x40);		// Start Display Line = 0
	lcd_writecom(0xA0);		// ADC = normal (Anzeigerichtung nicht gespiegelt)
	lcd_writecom(0xA3);		// LCD-Bias = 1/7 fr 4,5-5,5V    1/9 fr 3,3-4,5V
	lcd_writecom(0xC0);		// Output Mode = nicht gespiegelt
	lcd_writecom(0x2F);		// LCD-Spannungserzeugung/Regelung einschalten
	lcd_writecom(0x20);		// V5 Regulator Resistor Ratio = 0
	lcd_writecom(0xAC);		// Static Indicator off
	lcd_writecom(0x81);		// Kontrasteinstellung auswhlen
	lcd_writecom(32);		// Kontrast = mittel (fr Bias 1/7: 20-50, fr 1/9: 2-10)
	lcd_writecom(0xA4);		// Displaytest aus
	lcd_writecom(0xAF);		// Display ein
	lcd_writecom(0xA6);		// Display normal (nicht invertiert)
	lcd_clear();
}

void lcd_clear(void)
{unsigned char x, y;
	for (y=0;y<8; y++)
	{lcd_setadress(0,y);
		for (x=0;x<128; x++)
			lcd_writebyte(0);
	}
    lcd_setadress(0,0);
}

void lcd_setadress(unsigned char Spalte, unsigned char Zeile)    //Set Adresspointer
{   
	lcd_writecom(0xB0|Zeile);
	lcd_writecom(0x10|(Spalte/16));
	lcd_writecom(Spalte&15);
}

void lcd_BCD (unsigned char zahl)
{   lcd_writechar(48+zahl/100);
    lcd_writechar(48+(zahl/10)%10);
    lcd_writechar(48+zahl%10);
}

void lcd_BCDint (unsigned int zahl)
{   lcd_writechar(48+zahl/10000);
    lcd_writechar(48+(zahl/1000)%10);
    lcd_writechar(48+(zahl/100)%10);
    lcd_writechar(48+(zahl/10)%10);
    lcd_writechar(48+zahl%10);
}